import 'dart:io';

import 'package:flutter/material.dart';
import 'package:infinite_scroll_pagination/infinite_scroll_pagination.dart';
import 'package:local_image_pager/local_image_pager.dart';

/// GridView's crossAxisCount.
const _cross_axis_count = 3;

/// GridView's mainAxisSpacing, crossAxisSpacing, and padding. (Used to calculate the thumbnail size.)
const double _spacing = 1;

class GridPicParts extends StatefulWidget {
  const GridPicParts(this.itemSize, this.cacheSize) : super();
  final double itemSize;
  final int cacheSize;

  @override
  _GridPicPartsState createState() => _GridPicPartsState();
}

class _GridPicPartsState extends State<GridPicParts> {
  // ページング.
  final PagingController<int, String> _pagingController =
      PagingController(firstPageKey: 0);
  static const _num_per_load = 10;

  final pager = LocalImagePager();

  @override
  void initState() {
    super.initState();

    _pagingController
      ..addPageRequestListener(_fetchPage)
      ..addStatusListener((status) {
        if (status == PagingStatus.subsequentPageError) {
          Scaffold.of(context).showSnackBar(
            SnackBar(
              content: const Text(
                'Something went wrong while fetching a new page.',
              ),
            ),
          );
        }
      });
  }

  Future<void> _fetchPage(pageKey) async {
    try {

      final lastIndex = pageKey + _num_per_load - 1;
      final images =
      await pager.latestImages(pageKey, lastIndex);

      print('_fetchPage:$pageKey:${images.length}');

      if (images.length < _num_per_load) {
        _pagingController.appendLastPage(images);
      } else {
        final nextPageKey = lastIndex + 1;
        _pagingController.appendPage(images, nextPageKey);
      }
    } catch (error) {
      print('tab_my_page:_fetchPage:${error.toString()}');
      // _pagingController.error = error;
    }
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return
      PagedGridView<int, String>(
                                scrollDirection: Axis.horizontal,
                                gridDelegate:
                                    const SliverGridDelegateWithFixedCrossAxisCount(
                                        crossAxisCount:
                                            _cross_axis_count,
                                        mainAxisSpacing: _spacing,
                                        crossAxisSpacing: _spacing),
          pagingController: _pagingController,
          builderDelegate: PagedChildBuilderDelegate<String>(
            itemBuilder: (context, item, index)
            {
              print('itemBuilder:$index:$item');
            return InkWell(
              child: item == null ? Container() : Image.file(
                File(item),
                fit: BoxFit.cover,
                height: widget.itemSize,
                cacheHeight: widget.cacheSize,
              ),
              onTap: () {
                // _cropImage(path, context);
              },
            );
          },
            firstPageErrorIndicatorBuilder: (context) => const Text('画像取得に失敗しました'),
            newPageErrorIndicatorBuilder: (context) => const Text('画像取得に失敗しました'),
          )
      );
  }
}
